#include <LnxTools.h>

void INIT_EVENT( EVENT_HNDL* pEvent )
{
	pEvent->SetFlag = 0;
	pEvent->WaitCnt = 0;
	init_waitqueue_head( &pEvent->WaitQue);
	INIT_SPIN_LOCK( &pEvent->FlagLock );
}

int WAIT_EVENT( EVENT_HNDL* pEvent, ULONG msecDelay )
{
	int TimeRemain;
	unsigned long flags ;

	// wait on the queue
	TimeRemain = (msecDelay * HZ) / 1000;

	// if no wait time then just return failure
	if ( ! msecDelay )
	{
		return 0;
	}

	// atomically sleep if event is not set - we must protect the window between
	// finding out if the flag is already set and sleeping as if an interrupt
	// occurs in the window and calls SET_EVENT, SET_EVENT will signal the wait
	// queue before we get placed on it and we will miss this signal.
	save_flags (flags) ;
	cli () ;

	// indicate that I am waiting
	pEvent->WaitCnt++;

	if ( ! pEvent->SetFlag )
	{
		while ( !pEvent->SetFlag && TimeRemain )
		{
			// safe to call sleep with interrupts off as kernel will re-enable
			// interrupts after placing us on the queue and before calling schedule
			TimeRemain = interruptible_sleep_on_timeout ( &pEvent->WaitQue,  TimeRemain );
		}
	}
	// remove wait indication
	pEvent->WaitCnt--;

	restore_flags (flags) ;

	// return setflag
	return pEvent->SetFlag;
}

void SET_EVENT( EVENT_HNDL* pEvent )
{
	DWORD LockFlag;

	// if the event is cleared then
	// set the flag, wake up all waiting tasks.
	ACQUIRE_LOCK( &pEvent->FlagLock, LockFlag );
	if ( !pEvent->SetFlag )
	{
		int cnt;
		pEvent->SetFlag = TRUE;
		for  ( cnt=0; cnt<pEvent->WaitCnt; cnt++ )
		{
			wake_up_interruptible ( &pEvent->WaitQue  );
		}
	}
	RELEASE_LOCK( &pEvent->FlagLock, LockFlag );
}

void SET_EVENT_FROM_ISR( EVENT_HNDL* pEvent )
{
	// if the event is cleared then
	// set the flag, wake up all waiting tasks.
	ACQUIRE_LOCK_AT_ISR( &pEvent->FlagLock );
	if ( !pEvent->SetFlag )
	{
		int cnt;
		pEvent->SetFlag = TRUE;
		for  ( cnt=0; cnt<pEvent->WaitCnt; cnt++ )
		{
			wake_up_interruptible ( &pEvent->WaitQue  );
		}
	}
	RELEASE_LOCK_AT_ISR( &pEvent->FlagLock );
}

void RESET_EVENT( EVENT_HNDL* pEvent )
{
	DWORD LockFlag;

	// clear the event flag
	ACQUIRE_LOCK( &pEvent->FlagLock, LockFlag );
	pEvent->SetFlag = FALSE;

	// empty out the queue
	while ( waitqueue_active( &pEvent->WaitQue ) )
		interruptible_sleep_on( &pEvent->WaitQue );

	RELEASE_LOCK( &pEvent->FlagLock, LockFlag );
}





